@@ -1,3 +1,6 @@
+/// The NNModule trait
+mod nnmodule;
+pub use nnmodule::*;
 /// Linear Layers
 mod linear;
 pub use linear::*;
@@ -6,7 +9,6 @@ pub use linear::*;
 pub use sequential::*;
 /// Transformer Layers
 mod transformer;
-use tch::nn::Module;
 pub use transformer::*;
 /// RNN Layers
 mod rnn;
@@ -15,11 +17,4 @@ pub use rnn::*;
 mod activation;
 pub use activation::*;
 /// Module tests
-mod tests;
-
-/// A trait for some basic functions a module should have
-pub trait NNModule: Module {
-    fn train(&mut self);
-    fn eval(&mut self);
-    fn count_parameters(&self) -> u64;
-}
\ No newline at end of file
+mod tests;
\ No newline at end of file
